#include "entity.h"

#include <cmath>
#include "SDL_opengl.h"

#include "random.h"
#include "util.h"
#include "container.h"
#include "game.h"
#include "config.h"

namespace PRJID {

Entity::Entity() {
	init();
}

void Entity::init() {
	container_=NULL;
	myticks=0;
	type=ET_NONE;
	speed=1.0f;

	m_forward=false;
	m_backward=false;
	m_left=false;
	m_right=false;
}
C3f Entity::get_pos() { return position.get_position(); }
C3f Entity::get_rot() { return position.get_rotation(); }
void Entity::set_pos(C3f p) { position.set_position(p); }

void Entity::draw() {
	glPushMatrix();
	C3f pos;
	pos=get_pos();
	glTranslatef(pos.x,pos.y,pos.z);

	C3f rot;
	rot=get_rot();
	glRotatef(rot.x,-90.0f,0.0f,0.0f);
	glRotatef(90-rot.y,0.0f,-90.0f,0.0f);
	glRotatef(rot.z,0.0f,0.0f,-90.0f);

	glPopMatrix();
}



void Entity::act(unsigned int ticks) {
}

bool Entity::trymove(C3f d) {
	if (d.sum()<0.01f) {
		return true;
	}
	C3f nextpos;
	nextpos=get_pos()+d;

	if (Tweak::collisiondetection_i()==1&&!container_->get_game()->isvalid(nextpos)) {
	position.set_movement(C3f());
		container_->get_game()->action(A_DAMAGE);
		return false;
	}

	position.set_position(nextpos);
	position.set_movement(C3f());

	return true;
}

void Entity::tick(unsigned int ticks) {
	myticks++;

	float fticks=static_cast<float>(ticks);
	fticks*=0.10f;
	fticks*=speed;
	if (m_forward) { position.move_forward(fticks); }
	if (m_backward) { position.move_backward(fticks); }
	position.restrict_movement(C3b(false,true,false));
	if (m_left) { position.move_left(fticks); }
	if (m_right) { position.move_right(fticks); }
	position.restrict_movement(C3b(false,false,false));

	m_forward=false;
	m_backward=false;
	m_left=false;
	m_right=false;

	C3f dir;

	dir=position.get_movement();


	if (trymove(dir)) { return; }
}




void Entity::mark_move_right() { m_right=true; }
void Entity::mark_move_left() { m_left=true; }
void Entity::mark_move_forward() { m_forward=true; }
void Entity::mark_move_backward() { m_backward=true; }

void Entity::turn_right() { 
	position.rotate(0,-0.1f);
}

void Entity::turn_left() { 
	position.rotate(0,0.1f);
}
void Entity::turn_up() {
	position.rotate(0.1f,0);
}
void Entity::turn_down() {
	position.rotate(-0.1f,0);
}
void Entity::turn(int x,int y) {
	position.rotate(
	(static_cast<float>(x))/5.0f,
	((static_cast<float>(y))/5.0f)*Config::mouse_reverse()
	);
}

} // namespace
