#include <sstream>
#include "SDL_opengl.h"

#include "game.h"
#include "util.h"
#include "random.h"
#include "entity.h"
#include "help.h"

#include "tweak/tweak.h"
#include "config.h"

namespace PRJID {


Game::~Game() {
	map.del_dl();
}

static const unsigned int lastlevel=6;

Game::Game(Game *g,E_Diffi diffi) {
	container_.set_game(this);
	player_=container_.create(ET_PLAYER);
	ticks_=0;

	level=1;
	if (g) {
		level=g->get_level()+1;
	}

	state=GS_PLAYING;
	drawmap=true;
	timeleft=Timeleft(Tweak::timeleft_i());
	timeleft.dim(
		C3(250,0,0),
		C3f(1,5,0),
		C3f(10,10,0)	
	);
	energy.dim(
		C3(0,250,0),
		C3f(1,5,0),
		C3f(10,30,0)	
	);
	fin=Timeleft(Tweak::fin_i());
	startmessage=Timeleft(Tweak::startmessage_i());
	
	particleinterval=Interval(Tweak::particle_interval_i());

	lastparticle=0;
	switch (level) {
		case 1: 
			map.init("level_easy1"); 
			player_->set_pos(C3f(40,40,40));
			break;
		case 2: 
			map.init("level_easy2"); 
			player_->set_pos(C3f(23.8332,40.7511,39.2292));
			break;
		case 3: 
			map.init("level_stair"); 
			player_->set_pos(C3f(31.4259,51.8435,46.8324));
			break;
		case 4: 
			map.init("level_atrium"); 
			player_->set_pos(C3f(38.1786,18.0645,45.3965));
			break;
		case 5:
			map.init("level_rooms"); 
			player_->set_pos(C3f(16.2624,15.1276,14.3564));
			break;
		case 6:
			map.init("level_so"); 
			player_->set_pos(C3f(36.9157,63.5921,68.64));
			break;
		default:
			map.init("save"); 
			break;
	}
	map.gen_dl();

	cap();
}

void Game::addparticle() {
	Blockaddress rand=map.random();
	if (!rand.get_current()->is_active()) return;
	unsigned int side;
	side=Random::sget(6);
	C3f p;

	switch (side) {
		case 0: p.x=Random::sgetf(); p.y=1; p.z=Random::sgetf(); break;
		case 1: p.x=Random::sgetf(); p.y=0; p.z=Random::sgetf(); break;
		case 2: p.x=0; p.y=Random::sgetf(); p.z=Random::sgetf(); break;
		case 3: p.x=1; p.y=Random::sgetf(); p.z=Random::sgetf(); break;
		case 4: p.x=Random::sgetf(); p.y=Random::sgetf(); p.z=0; break;
		case 5: p.x=Random::sgetf(); p.y=Random::sgetf(); p.z=1; break;
	}

	p=rand.scale_to_level(p);

	C3f f;
	f=p;

	C3f rp;
	rp=rand.get_pos();

	f+=rp;

	particles.add(f);
}

int Game::action(E_Action id,int x,int y) {
	switch (id) {
		case A_FORWARD: player_->mark_move_forward(); break;
		case A_BACKWARD: player_->mark_move_backward(); break;
		case A_LEFT: player_->mark_move_left(); break;
		case A_RIGHT: player_->mark_move_right(); break;

		case A_TURNLEFT: player_->turn_left(); break;
		case A_TURNRIGHT: player_->turn_right(); break;

		case A_TURN: player_->turn(x,y); break;
		case A_DAMAGE: 
			energy.reduce(Tweak::collisioncost_i());
			Audio::play(AS_COLLISION);
			break;
		case A_ADDPARTICLE:
			addparticle();
			break;
		default:
			break;
	}
	return 0;
}

bool Game::inprogress() {
	if (state==GS_PLAYING) return true;
	if (state==GS_FAILED) return true;
	if (state==GS_FINISHED) return true;
	return false;
}

void Game::cap() {

	C3f pos;
	pos=player_->get_pos();
	if (
		pos.exceeds(0, Tweak::blockscale_f() )&&
		Tweak::playmode_i()
	) {
		state=GS_FINISHED;
		Audio::schedule_play(2000,AS_PASSED,AC_TITLE);
		fin.reset();
	}
	if (timeleft.over()&&Tweak::playmode_i()) {
		Audio::schedule_play(2000,AS_FAILED,AC_TITLE);
		state=GS_FAILED;
		fin.reset();
	}
	if (energy.empty()&&Tweak::playmode_i()) {
		Audio::schedule_play(2000,AS_FAILED,AC_TITLE);
		state=GS_FAILED;
		fin.reset();
	}


}

void Game::tick(unsigned int ticks) {
	ticks_+=ticks;
	fin.tick(ticks);
	if (state!=GS_PLAYING) return;

	for (unsigned int i=0;i<ticks;i++) {
		container_.act(ticks);
		container_.tick(ticks);
	}

	if (particleinterval.check(ticks_)) {
		for (int i=0;i<Tweak::particle_count_i();i++) {
			action(A_ADDPARTICLE);
		}
	}

	timeleft.tick(ticks);
	energy.tick(ticks);
	startmessage.tick(ticks);
	cap();
}


void Game::rotrans() {
	glLoadIdentity();

	player_->scene_rot();
	player_->scene_trans();
}


void Game::draw() {
	Frustum::instance()->stat_reset();
	glEnable(GL_LIGHTING);

	rotrans();
	if (Tweak::wireframe_i()) {
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	} else {
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	}

	Frustum::instance()->calculate();

	if (Tweak::debugdraw_i()) {
		map.debug_draw(drawmap);
	} else {
		map.draw();
	}
	particles.draw();

	ortho2d();
	timeleft.draw();
	energy.draw();
	if (state==GS_FINISHED&&fin.over()) {
		if (level==lastlevel) {
			Help::end();
		} else {
			Help::fin();
		}
	}
	if (state==GS_FAILED&&fin.over()) {
		Help::gameover();
	}
	if (!startmessage.over()) {
		if (level==1) { if (ismouselocked()) { Help::mouselock_off(); } else { Help::mouselock(); } }
	}
	ortho2d_off();
}

bool Game::handle_event(SDL_Event event) {
	return handle_game_event(event);
}


bool Game::isvalid(C3f p) {
	return !map.does_collide(p,0.12);
}


bool Game::handle_game_event(SDL_Event event) {
	switch (event.type) {
		case SDL_QUIT: return false; break;
		case SDL_VIDEORESIZE: return false; break;
		case SDL_MOUSEBUTTONDOWN:
			if (Tweak::editmode_i()==1)  {
				map.select( event.button.x, Config::dy()-event.button.y);
				break;
			}
			if (!ismouselocked()) { lockmouse(); }
			if (state==GS_FINISHED&&fin.over()) {
				state=GS_NEXTLEVEL;
				if (level==lastlevel) state=GS_QUIT;
				break;
			}
			if (state==GS_FAILED&&fin.over()) {
				state=GS_GAMEOVER;
				break;
			}
			break;
		case SDL_MOUSEMOTION:
			if (!ismouselocked()) break;
			action(A_TURN,event.motion.xrel,event.motion.yrel);
			break;
		case SDL_KEYDOWN:
			if (state==GS_FINISHED&&fin.over()) {
				state=GS_NEXTLEVEL;
				if (level==lastlevel) state=GS_QUIT;
				break;
			}
			if (state==GS_FAILED&&fin.over()) {
				state=GS_GAMEOVER;
				break;
			}
			switch (event.key.keysym.sym) {
				case SDLK_ESCAPE: state=GS_QUIT; break;


				//debug stuff
				case SDLK_1: map.toggle_selected(); break;
				case SDLK_2: map.split_selected(); break;
				case SDLK_3: map.merge_selected(); break;
				case SDLK_4: map.check(); break;
				case SDLK_p: action(A_ADDPARTICLE); break;
				case SDLK_l:  state=GS_NEXTLEVEL; break;

				case SDLK_F2: map.save(); break;
				case SDLK_F4: map.load(); break;
				case SDLK_c: std::cout << "coll: " << map.does_collide(player_->get_pos(),0.1) << std::endl; break;
				case SDLK_x: Frustum::instance()->stat_print(); break;
				case SDLK_y: {C3f p; p=player_->get_pos(); std::cout << "pos: " <<  p << std::endl; }break;


				default:
					return false;
					break;
			}
			break;
		default:
			return false;
	}
	return true;
}
void Game::handle_state() {
	Uint8 *keys;
	keys = SDL_GetKeyState(NULL);

	if (keys[SDLK_a]) {action(A_LEFT); }
	if (keys[SDLK_LEFT]) {action(A_LEFT); }
	if (keys[SDLK_d]) {action(A_RIGHT); }
	if (keys[SDLK_RIGHT]) {action(A_RIGHT); }

	if (keys[SDLK_w]) {action(A_FORWARD); }
	if (keys[SDLK_s]) {action(A_BACKWARD); }
	if (keys[SDLK_UP]) {action(A_FORWARD); }
	if (keys[SDLK_DOWN]) {action(A_BACKWARD); }

}

std::ostream& operator<<(std::ostream&s, Game sk) {
	return s;
}

} //namespace
