#include "SDL.h"
#include "SDL_opengl.h"

#include <iostream>

#include "release.h"
#include "game.h"
#include "util.h"
#include "audio.h"
#include "random.h"
#include "cube.h"
#include "help.h"
#include "font_ogl.h"
#include "config.h"
#include "map.h"

#include "tweak/tweak.h"

namespace PRJID {

void light() {
    GLfloat light0_pos[4]   = { 0.0, 5.0, .0, 0.0 };
    GLfloat light0_color[4] = { .6, .6, .6, 1.0 }; 

    GLfloat light1_pos[4]   = {  20.0, 5.0, 20.0, 0.0 };
    GLfloat light1_color[4] = { .4, .4, 1, 1.0 };  

    glShadeModel(GL_SMOOTH);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);

    glLightfv(GL_LIGHT0, GL_POSITION, light0_pos);
    glLightfv(GL_LIGHT0, GL_AMBIENT,  light0_color);
    glLightfv(GL_LIGHT1, GL_POSITION, light1_pos);
    glLightfv(GL_LIGHT1, GL_DIFFUSE,  light1_color);
    glEnable(GL_LIGHT0);
    glEnable(GL_LIGHT1);
    glEnable(GL_LIGHTING);


	glClearColor(.1,.1,1,1);

}
std::string resource_path(std::string dir) {
	bool err;

	std::string path;
	if (dir=="") {
		path="data/";
		std::cout << "  Looking in " << path << std::endl;
		err=Audio::load(path);
		if (err) return path;
		std::cout << "    Not found " << std::endl;

		path="/usr/local/share/";
		path+=PATHNAME;
		path+="/";
		std::cout << "  Looking in " << path << std::endl;
		err=Audio::load(path);
		if (err) return path;
		std::cout << "    Not found " << std::endl;

		path="/usr/share/";
		path+=PATHNAME;
		path+="/";
		std::cout << "  Looking in " << path << std::endl;
		err=Audio::load(path);
		if (err) return path;
		std::cout << "    Not found " << std::endl;
	}

	std::cout << "  Looking in " << dir << std::endl;
	err=Audio::load(dir);
	if (err) return dir;
		std::cout << "    Not found " << std::endl;

	std::cerr << "  Resources not found! " << std::endl;
	return "";
}

bool testspeed() {
	Uint32 start,end,took;

	start=SDL_GetTicks();
	ortho2d();
	glColor3f(1,1,1);

	Font_ogl::write("loading...");
	SDL_GL_SwapBuffers();
	ortho2d_off();
	end=SDL_GetTicks();

	took=(end-start);

	if (took<600) return true;

   SDL_Event event;
   while (true) {
		glLoadIdentity();
     	while  (SDL_PollEvent(&event)) {
			if (event.type==SDL_QUIT) return false;
			if (event.type==SDL_MOUSEBUTTONDOWN) return true;
			if (event.type!=SDL_KEYDOWN) continue;
			if (event.key.keysym.sym==SDLK_ESCAPE) { return false; }
			return true;
		}
		ortho2d();
		glTranslatef(0,20,0);
		Font_ogl::write("  ... system slow");
		glTranslatef(0,20,0);
		Font_ogl::write("  probable cause: no properly configured 3d acceleration.");
		glTranslatef(0,20,0);
		Font_ogl::write("  ESC to quit. any other key: continue anyway.");
		ortho2d_off();
		SDL_GL_SwapBuffers();
		SDL_Delay(150);
	}
	return true;
}

SDL_Surface *screen;

bool setmode(int w,int h,bool fullscreen) {
	if(!(screen=SDL_SetVideoMode(w,h,0,SDL_OPENGL|SDL_RESIZABLE | ((fullscreen) ? SDL_FULLSCREEN : 0)))) {
		std::cerr <<"SDL_SetVideoMode error " << w << "x" << h << " " << fullscreen << "  " << SDL_GetError() << std::endl;
		return false;
	}
	Config::resolution(w,h);
	SDL_WM_SetCaption(FULLNAME,"");
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   float fov;
   fov=Tweak::fov_f();
   gluPerspective(fov,static_cast<GLfloat>(screen->w)/static_cast<GLfloat>(screen->h),0.10f,250.0f);
   glViewport(0, 0, screen->w, screen->h);
   glMatrixMode(GL_MODELVIEW);

   glEnable(GL_DEPTH_TEST);
	return true;
}

typedef enum e_menuids {
	M_POP=-1,
	M_NONE=0,
	M_START_EASY=1,
	M_START_MEDIUM=2,
	M_START_HARD=3,
	M_START_NIGHTMARE=4,
	M_QUIT=5
};


int main(int w,int h,bool fullscreen,bool audio,std::string dir) {
   std::cout<<std::endl<<std::endl<<FULLNAME << " "<< VERSION << std::endl;
   std::cout<<"\t Copyright (C) 2006" << std::endl;
   std::cout<<"\t " URL1 << std::endl;
   std::cout<<"\t Using libSDL,SDL_mixer under the terms of the GNU LGPL" << std::endl;
	std::cout << std::endl;
   /*std::cout<<"This program is free software; you can redistribute it and/or"<<std::endl;
   std::cout<<"modify it under the terms of the GNU General Public License," <<std::endl;
   std::cout<<"version 2, as published by the Free Software Foundation; You"<<std::endl;
   std::cout<<"should have received a copy of the GNU General Public License"<<std::endl;
   std::cout<<"along with this program; if not, write to the Free Software" <<std::endl;
   std::cout<<"Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA."<<std::endl;
   std::cout << std::endl << std::endl;*/
	Tweak::tick();

	if(SDL_Init(SDL_INIT_VIDEO)==-1) {
		std::cerr <<"SDL_Init error SDL_INIT_VIDEO" << std::endl;
		return EXIT_FAILURE;
	}
	atexit(SDL_Quit);
	if (!setmode(w,h,fullscreen)) {
		std::cerr <<"SDL_SetVideoMode error" << std::endl;
		return EXIT_FAILURE;
	}


	std::cout << "Uploading Font. " << std::endl;
	Font_ogl::genlist();
	error();

	std::cout << "Testing speed: " << std::endl;
	if (!testspeed()) { return EXIT_SUCCESS; }
	std::cout << "Starting audio subsys. " << std::endl;
	if (audio) {
		if (!Audio::init()) {
			std::cerr << "  Error in Mix_OpenAudio" << std::endl;
		}
		std::cout << "Loading audio: " << std::endl;
		dir=resource_path(dir);
		//error();
	} else { Audio::off(); }

	Map::set_path(dir);
	if (!Particles::load(dir)) {
	}

	std::cout << "Uploading Models: " << std::endl;
	Cube::gen_dl();
	error();

	Audio::play(AS_TITLE,AC_TITLE);
	light();

	Random::sseed();

	Game *game=NULL;

   SDL_Event event;
	int timestamp,oldtimestamp=0;

	bool handled;

	unsigned int unhandled=0;
	unsigned int showunhandled=Tweak::helptime_i();

	unsigned int delay=1;
	unsigned int record=0;

	game=new Game(NULL,D_EASY); 

   while (true) {

     	while  (SDL_PollEvent(&event)) {
			handled=false;
			if (game) { 
				handled=game->handle_event(event); 
			}
			if (!handled) switch (event.type) {
					case SDL_VIDEORESIZE:
						setmode(event.resize.w,event.resize.h,false);
						break;
      			case SDL_QUIT: return EXIT_SUCCESS; break;

					case SDL_KEYDOWN: switch (event.key.keysym.sym) {
						case SDLK_w:
						case SDLK_a:
						case SDLK_s:
						case SDLK_d:
						case SDLK_LEFT:
						case SDLK_RIGHT:
						case SDLK_UP:
						case SDLK_DOWN:
						case SDLK_ESCAPE:
							break;

						case SDLK_F1:
						case SDLK_h:
							if (showunhandled>0) {showunhandled=0; break; }
							unhandled=6; break;

						case SDLK_n: 
							delete game;
							game=new Game();
							break;
						case SDLK_r: Config::toggle_mouse_reverse(); break;
						case SDLK_m: togglemouselock(); break;
						case SDLK_f: SDL_WM_ToggleFullScreen(screen); break;
						case SDLK_F10: screenshot(screen->w,screen->h); break;
						case SDLK_F11: Config::toggle_record(); record=0; break;
						case SDLK_v: Audio::toggle(); break;
						default: 
							//unhandled key
							unhandled++;
							break;
					}
			}
		}
		if (Random::sget(30)==10) Tweak::tick();


		glLoadIdentity();
		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
		if (game) {
			game->handle_state();


			int diff;
			timestamp=SDL_GetTicks();
			diff=timestamp-oldtimestamp;
			oldtimestamp=timestamp;



			game->tick(diff);


			game->draw();
			if (!game->inprogress()) {
				E_GameState state=game->get_state();
				if (state==GS_NEXTLEVEL) {
					Game *newgame=new Game(game);
					delete game;
					game = newgame;
				} else if (state==GS_QUIT) {
					return EXIT_SUCCESS;
				} else {
					delete game;
					game=NULL;
				}
			}
			if (unhandled>5) {
				showunhandled=Tweak::helptime_i();
				unhandled=0;
			}

			if (showunhandled>0) {
				showunhandled--;
				ortho2d();
				glDisable(GL_LIGHTING);
				Help::controls();
				Help::gameplay();
				glEnable(GL_LIGHTING);
				ortho2d_off();
			}
			delay=1;
		} else {
			game=new Game(NULL,D_EASY); 
		}


		SDL_GL_SwapBuffers();
		if (Config::record()) {
			record++;
			screenshot(screen->w,screen->h,record);
		}

		SDL_Delay(delay);
		Audio::tick();
		error();

	}
}

} // namespace

int main(int argc,char *argv[]) {
	unsigned int w=800; unsigned int h=600;
	bool fullscreen=false;
	bool audio=true;
	std::string dir="";

	std::istringstream istr;
	if (argc>1) {
		for (int i=1;i<argc;i++) {
			std::string arg(argv[i]);

			if (arg=="-fullscreen") {
				fullscreen=true;
				continue;
			} 
			if (arg=="-noaudio") {
				audio=false;
				continue;
			} 
			if (arg=="-width") {
				if (i==argc-1) {
					std::cerr << "Argument missing for -width " << std::endl;
					return EXIT_FAILURE;
				}
				istr.clear();
				istr.str(argv[i+1]);
				istr >> w;
				i++;
				continue;
			} 
			if (arg=="-height") {
				if (i==argc-1) {
					std::cerr << "Argument missing for -height " << std::endl;
					return EXIT_FAILURE;
				}
				istr.clear();
				istr.str(argv[i+1]);
				istr >> h;
				i++;
				continue;
			} 
			if (arg=="-dir") {
				if (i==argc-1) {
					std::cerr << "Argument missing for -dir " << std::endl;
					return EXIT_FAILURE;
				}
				dir=argv[i+1];
				i++;
				continue;
			}
			std::cerr << "Usage: " << argv[0] << " [ -width xx ] [ -height xx ] [ -fullscreen ] [ -noaudio ] [ -dir datadir ]" << std::endl;
			if (arg!="-help") {
				std::cerr << "Unrecognized argument " << arg << std::endl;
				return EXIT_FAILURE;
			}
			return EXIT_SUCCESS;
		}
	}

	int r;
	r=PRJID::main(w,h,fullscreen,audio,dir);

	if (r==EXIT_SUCCESS) {
		std::cout << std::endl << "   Thank you for playing " << FULLNAME << "." << std::endl;
	}
	return r;
}
